/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>

#include "gimageview.h"

#include "charset.h"
#include "comment.h"
#include "gtk2-compat.h"
#include "gtkutils.h"
#include "prefs.h"
#include "prefs_ui_etc.h"
#include "prefs_win.h"
#include "thumbnail_support.h"


typedef struct PrefsWin_Tag {
   /* cache */
   GtkWidget *cache_read_clist1;
   GtkWidget *cache_read_clist2;
   GtkWidget *cache_read_add;
   GtkWidget *cache_read_del;
   GtkWidget *cache_read_up;
   GtkWidget *cache_read_down;
   gint       cache_read_clist1_selected;
   gint       cache_read_clist2_selected;
   GtkWidget *cache_write_type;
   GtkWidget *cache_write_prefs;

   /* comment */
   GtkWidget *comment_clist;
   GtkWidget *comment_key_entry;
   GtkWidget *comment_name_entry;
   GtkWidget *comment_status_toggle;
   GtkWidget *comment_auto_toggle;
   GtkWidget *comment_disp_toggle;
   GtkWidget *comment_new;
   GtkWidget *comment_add;
   GtkWidget *comment_change;
   GtkWidget *comment_del;
   gint       comment_selected;
} PrefsWin;

static PrefsWin  prefs_win;
extern Config   *config_changed;
extern Config   *config_prechanged;



static void
cb_colorsel_ok (GtkWidget *button, gboolean *retval)
{
   *retval = TRUE;
   gtk_main_quit ();
}


static void
cb_colorsel_cancel (GtkWidget *button, gboolean *retval)
{
   *retval = FALSE;
   gtk_main_quit ();
}


static gint
cb_dummy (GtkWidget *button, gpointer data)
{
   return TRUE;
}


void
cb_choose_color (GtkWidget *widget, gint color[])
{
   GtkWidget *dialog, *button, *csel;
   gboolean retval = FALSE;
   gdouble selcol[4];

   g_return_if_fail (color);

   dialog = gtk_color_selection_dialog_new (_("Choose Color"));
   selcol[0] = color[0] / 0xffff;
   selcol[1] = color[1] / 0xffff;
   selcol[2] = color[2] / 0xffff;
   csel = GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel;     
   gtk_color_selection_set_color (GTK_COLOR_SELECTION (csel), selcol);
   gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
		       GTK_SIGNAL_FUNC (cb_dummy), NULL);

   button = GTK_COLOR_SELECTION_DIALOG (dialog)->ok_button;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (cb_colorsel_ok),
		       &retval);
   button = GTK_COLOR_SELECTION_DIALOG (dialog)->cancel_button;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (cb_colorsel_cancel),
		       &retval);
   button = GTK_COLOR_SELECTION_DIALOG (dialog)->help_button;
   gtk_widget_hide (button);
   gtk_widget_show (dialog);

   gtk_grab_add (dialog);
   gtk_main ();
   if (retval) {
      gtk_color_selection_get_color (GTK_COLOR_SELECTION (csel), selcol);
      color[0] = selcol[0] * 0xffff;
      color[1] = selcol[1] * 0xffff;
      color[2] = selcol[2] * 0xffff;
   }
   gtk_grab_remove (dialog);
   gtk_widget_destroy (dialog);
}


static void
cache_read_column_add (gchar *label)
{
   GtkWidget *clist1 = prefs_win.cache_read_clist1;
   GtkWidget *clist2 = prefs_win.cache_read_clist2;
   gint row, can_retrieve = 1;
   gchar *text;

   g_return_if_fail (label && label[0] != '\0');

   for (row = 0; can_retrieve; row++) {
      can_retrieve = gtk_clist_get_text (GTK_CLIST (clist1), row, 0, &text);
      if (!can_retrieve) break;

      if (text && !strcmp (label, text)) {
	 gtk_clist_append (GTK_CLIST (clist2), &text);
	 gtk_clist_remove (GTK_CLIST (clist1), row);
	 return;
      }
   }
}


static void
cache_read_column_del (gchar *label)
{
   GtkWidget *clist1 = prefs_win.cache_read_clist1;
   GtkWidget *clist2 = prefs_win.cache_read_clist2;
   gint row, can_retrieve = 1;
   gchar *text;

   g_return_if_fail (label && label[0] != '\0');

   for (row = 0; can_retrieve; row++) {
      can_retrieve = gtk_clist_get_text (GTK_CLIST (clist2), row, 0, &text);
      if (!can_retrieve) return;

      if (text && !strcmp (label, text)) {
	 gtk_clist_append (GTK_CLIST (clist1), &text);
	 gtk_clist_remove (GTK_CLIST (clist2), row);
	 return;
      }
   }
}


static void
cache_read_set_data (gchar *string)
{
   gchar **labels;
   gint i;

   if (!string) return;

   labels = g_strsplit (string, ",", -1);

   for (i = 0; labels[i]; i++) {
      cache_read_column_add (labels[i]);
   }

   g_strfreev (labels);
}


static void
cache_read_update_config ()
{
   GtkWidget *clist2 = prefs_win.cache_read_clist2;
   gint row, can_retrieve = 1;
   gchar *string = NULL, *tmpstr, *coltext;

   for (row = 0; can_retrieve; row++) {
      can_retrieve = gtk_clist_get_text (GTK_CLIST (clist2), row, 0, &coltext);
      if (!can_retrieve) break;

      if (string) {
	 tmpstr = string;
	 string = g_strconcat (string, ",", NULL);
	 g_free (tmpstr);
      } else {
	 string = g_strdup ("");
      }
      tmpstr = string;
      string = g_strconcat (tmpstr, coltext, NULL);
      g_free (tmpstr);
   }

   if (config_changed->cache_read_list
       != config_prechanged->cache_read_list)
   {
      g_free (config_changed->cache_read_list);
   }
   config_changed->cache_read_list = string;
}


static void
set_sensitive_cache_read ()
{
   GtkWidget *clist1 = prefs_win.cache_read_clist1;
   GtkWidget *clist2 = prefs_win.cache_read_clist2;
   gint selected1 = prefs_win.cache_read_clist1_selected;
   gint selected2 = prefs_win.cache_read_clist2_selected;
   gint rownum1, rownum2;

   g_return_if_fail (clist1 && clist2);

   rownum1 = GTK_CLIST (clist1)->rows;
   rownum2 = GTK_CLIST (clist2)->rows;

   /* add button */
   if (selected1 < 0 || selected1 >= rownum1) {
      gtk_widget_set_sensitive (prefs_win.cache_read_add, FALSE);
   } else {
      gtk_widget_set_sensitive (prefs_win.cache_read_add, TRUE);
   }

   /* delete button */
   if (selected2 < 0 || selected2 >= rownum2) {
      gtk_widget_set_sensitive (prefs_win.cache_read_del, FALSE);
   } else {
      gtk_widget_set_sensitive (prefs_win.cache_read_del, TRUE);
   }

   /* up button */
   if (selected2 > 0 && selected2 < rownum2) {
      gtk_widget_set_sensitive (prefs_win.cache_read_up, TRUE);
   } else {
      gtk_widget_set_sensitive (prefs_win.cache_read_up, FALSE);
   }

   /* down button */
   if (selected2 >= 0 && selected2 < rownum2 - 1) {
      gtk_widget_set_sensitive (prefs_win.cache_read_down, TRUE);
   } else {
      gtk_widget_set_sensitive (prefs_win.cache_read_down, FALSE);
   }
}


static void
cb_clist_cache_read_select_row (GtkCList *clist, gint row, gint col,
				GdkEventButton *event, gpointer data)
{
   gint *selected = data;

   g_return_if_fail (clist);
   g_return_if_fail (data);

   *selected = row;

   set_sensitive_cache_read ();
}


static void
cb_clist_cache_read_unselect_row (GtkCList *clist, gint row, gint col,
				  GdkEventButton *event, gpointer data)
{
   gint *selected = data;

   g_return_if_fail (clist);
   g_return_if_fail (data);

   *selected = -1;

   set_sensitive_cache_read ();
}


static void
cb_cache_read_add_button_pressed (GtkButton *button, gpointer data)
{
   GtkWidget *clist1 = prefs_win.cache_read_clist1;
   gint row = prefs_win.cache_read_clist1_selected;
   gint can_retrieve;
   gchar *text;

   if (row < 0) return;

   can_retrieve = gtk_clist_get_text (GTK_CLIST (clist1), row, 0, &text);
   if (!can_retrieve || !text) return;

   cache_read_column_add (text);

   cache_read_update_config ();

   set_sensitive_cache_read ();
}


static void
cb_cache_read_del_button_pressed (GtkButton *button, gpointer data)
{
   GtkWidget *clist2 = prefs_win.cache_read_clist2;
   gint row = prefs_win.cache_read_clist2_selected;
   gint can_retrieve;
   gchar *text;

   if (row < 0) return;

   can_retrieve = gtk_clist_get_text (GTK_CLIST (clist2), row, 0, &text);
   if (!can_retrieve || !text) return;

   cache_read_column_del (text);

   cache_read_update_config ();

   set_sensitive_cache_read ();
}


static void
cb_cache_read_up_button_pressed (GtkButton *button, gpointer data)
{
   GtkWidget *clist = prefs_win.cache_read_clist2;
   gint selected = prefs_win.cache_read_clist2_selected;
   gint rownum;

   g_return_if_fail (button && clist);

   rownum = GTK_CLIST (clist)->rows;
   if (selected < 1 || selected > rownum - 1) return;

   gtk_clist_swap_rows (GTK_CLIST (clist), selected, selected - 1);
   prefs_win.cache_read_clist2_selected--;
   gtk_clist_moveto (GTK_CLIST (clist), selected - 1, 0, 0, 0);

   cache_read_update_config ();

   set_sensitive_cache_read ();
}


static void
cb_cache_read_down_button_pressed (GtkButton *button, gpointer data)
{
   GtkWidget *clist = prefs_win.cache_read_clist2;
   gint selected = prefs_win.cache_read_clist2_selected;
   gint rownum;

   g_return_if_fail (button && clist);

   rownum = GTK_CLIST (clist)->rows;
   if (selected < 0 || selected > rownum - 2) return;

   gtk_clist_swap_rows (GTK_CLIST (clist), selected, selected + 1);
   prefs_win.cache_read_clist2_selected++;
   gtk_clist_moveto (GTK_CLIST (clist), selected + 1, 0, 0, 0);

   cache_read_update_config ();

   set_sensitive_cache_read ();
}


void
set_sensitive_cache_write ()
{
   GtkWidget *option_menu = prefs_win.cache_write_type;
   GtkWidget *button = prefs_win.cache_write_prefs;
   GtkWidget *menu_item;
   gchar *label;

   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));

   menu_item = GTK_OPTION_MENU (option_menu)->menu_item;
   if (!menu_item) return;

   label = gtk_object_get_data (GTK_OBJECT (menu_item), "label");
   if (!label) return;

   if (thumbsupport_has_save_prefs (label))
      gtk_widget_set_sensitive (button, TRUE);
   else
      gtk_widget_set_sensitive (button, FALSE);
}


static void
cb_cache_write_type (GtkWidget *widget, gchar *data)
{
   if (!data) return;

   if (config_changed->cache_write_type != config_prechanged->cache_write_type)
      g_free (config_changed->cache_write_type);
   config_changed->cache_write_type = g_strdup (data);

   set_sensitive_cache_write ();
}


void
cb_cache_write_prefs_ok (GtkWidget *widget)
{
   g_return_if_fail (GTK_IS_WINDOW (widget));
   gtk_main_quit ();
}


static void
cb_cache_write_prefs_button_pressed (GtkWidget *option_menu)
{
   GtkWidget *dialog, *widget, *button, *menu_item;
   gchar *label;

   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));

   menu_item = GTK_OPTION_MENU (option_menu)->menu_item;
   if (!menu_item) return;

   label = gtk_object_get_data (GTK_OBJECT (menu_item), "label");
   if (!label) return;

   widget = thumbsupport_get_save_prefs (label, NULL);
   if (!widget) return;

   /* create dialog window */
   dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog), _("Preference - Cache Writing -")); 
   gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
   gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
		       GTK_SIGNAL_FUNC (cb_dummy), NULL);

   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
		       widget, TRUE, TRUE, 0);

   /* ok buttons */
   button = gtk_button_new_with_label (_("OK"));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
		       button, TRUE, TRUE, 0);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			      GTK_SIGNAL_FUNC (cb_cache_write_prefs_ok),
			      GTK_OBJECT (dialog));

   gtk_widget_show_all (dialog);

   gtk_grab_add (dialog);
   gtk_main ();
   gtk_grab_remove (dialog);
   gtk_widget_destroy (dialog);
}


static gchar *
boolean_to_text (gboolean bool)
{
   if (bool)
      return "TRUE";
   else
      return "FALSE";
}


static void
copy_comment_data_entry (CommentDataEntry *dest, CommentDataEntry *src)
{
   g_return_if_fail (dest);
   g_return_if_fail (src);

   *dest = *src;
   if (src->key)
      dest->key = g_strdup (src->key);
   if (src->display_name)
      dest->display_name = g_strdup (src->display_name);
}


static void
sync_to_row_data_comment (gint row)
{
   GtkCList *clist = GTK_CLIST (prefs_win.comment_clist);
   CommentDataEntry *entry;

   entry = gtk_clist_get_row_data (clist, row);
   if (!entry) return;

   gtk_clist_set_text (clist, row, 0, entry->key);
   if (entry->userdef) {
      gtk_clist_set_text (clist, row, 1, entry->display_name);
      gtk_clist_set_text (clist, row, 5, _("User defined"));
   } else {
      gtk_clist_set_text (clist, row, 1, _(entry->display_name));
      gtk_clist_set_text (clist, row, 5, _("System defined"));
   }
   gtk_clist_set_text (clist, row, 2, _(boolean_to_text (entry->enable)));
   gtk_clist_set_text (clist, row, 3, _(boolean_to_text (entry->auto_val)));
   gtk_clist_set_text (clist, row, 4, _(boolean_to_text (entry->display)));
}


static void
set_value_comment ()
{
   GtkCList *clist = GTK_CLIST (prefs_win.comment_clist);
   GtkToggleButton *t_ins  = GTK_TOGGLE_BUTTON (prefs_win.comment_status_toggle);
   GtkToggleButton *t_auto = GTK_TOGGLE_BUTTON (prefs_win.comment_auto_toggle);
   GtkToggleButton *t_disp = GTK_TOGGLE_BUTTON (prefs_win.comment_disp_toggle);
   GList *list;
   gchar *text[16];
   gint row;

   list = comment_get_data_entry_list ();

   while (list) {
      CommentDataEntry *dentry = list->data, *rowdata;

      list = g_list_next (list);

      if (!dentry) continue;

      rowdata = g_new0 (CommentDataEntry, 1);
      copy_comment_data_entry (rowdata, dentry);

      text[0] = dentry->key;
      text[1] = _(dentry->display_name);
      text[2] = _(boolean_to_text (dentry->enable));
      text[3] = _(boolean_to_text (dentry->auto_val));
      text[4] = _(boolean_to_text (dentry->display));
      if (dentry->userdef)
	 text[5] = _("User defined");
      else
	 text[5] = _("System defined");
      row = gtk_clist_append (clist, text);

      if (row >= 0) {
	 gtk_clist_set_row_data_full (clist, row, rowdata,
				      (GtkDestroyNotify) comment_data_entry_delete);
      } else {
	 comment_data_entry_delete (rowdata);
      }
   }

   gtk_toggle_button_set_active (t_ins,  TRUE);
   gtk_toggle_button_set_active (t_auto, FALSE);
   gtk_toggle_button_set_active (t_disp, TRUE);
}


static void
update_config_comment ()
{
   GtkCList *clist = GTK_CLIST (prefs_win.comment_clist);
   gchar *string, *tmp;
   gint row;

   if (config_changed->comment_key_list != config_prechanged->comment_key_list)
      g_free (config_changed->comment_key_list);
   config_changed->comment_key_list = NULL;

   string = g_strdup ("");

   for (row = 0; row < clist->rows; row++) {
      CommentDataEntry *entry;

      entry = gtk_clist_get_row_data (clist, row);     
      if (!entry) continue;
      if (!entry->key || !*entry->key) continue;
      if (!entry->display_name || !*entry->display_name) continue;

      tmp = string;
      if (*string) {
	 string = g_strconcat (string, "; ", NULL);
	 g_free (tmp);
	 tmp = string;
      }
      string = g_strconcat (string, entry->key, ",", entry->display_name,
			    ",", boolean_to_text (entry->enable),
			    ",", boolean_to_text (entry->auto_val),
			    ",", boolean_to_text (entry->display),
			    NULL);
      g_free (tmp);
      tmp = NULL;
   }

   config_changed->comment_key_list = string;
}


void
set_sensitive_comment ()
{
   GtkWidget *clist  = prefs_win.comment_clist;
   GtkWidget *t_auto = prefs_win.comment_auto_toggle;
   GtkWidget *key_entry  = prefs_win.comment_key_entry;
   GtkWidget *name_entry = prefs_win.comment_name_entry;
   GtkWidget *new    = prefs_win.comment_new;
   GtkWidget *add    = prefs_win.comment_add;
   GtkWidget *change = prefs_win.comment_change;
   GtkWidget *delete = prefs_win.comment_del;
   CommentDataEntry *entry;
   const gchar *text[2];
   gint row = prefs_win.comment_selected;

   text[0] = gtk_entry_get_text (GTK_ENTRY (key_entry));
   text[1] = gtk_entry_get_text (GTK_ENTRY (name_entry));

   if (row < 0) {
      gtk_widget_set_sensitive (key_entry,  TRUE);
      gtk_widget_set_sensitive (name_entry, TRUE);
      gtk_widget_set_sensitive (t_auto, FALSE);
      gtk_widget_set_sensitive (change, FALSE);
      gtk_widget_set_sensitive (delete, FALSE);
   } else {
      entry = gtk_clist_get_row_data (GTK_CLIST (clist), row);
      if (entry) {
	 gtk_widget_set_sensitive (key_entry,  entry->userdef);
	 gtk_widget_set_sensitive (name_entry, entry->userdef);
	 if (entry->def_val_fn)
	    gtk_widget_set_sensitive (t_auto, TRUE);
	 else
	    gtk_widget_set_sensitive (t_auto, FALSE);
	 if  (entry->userdef)
	    gtk_widget_set_sensitive (delete, TRUE);
	 else
	    gtk_widget_set_sensitive (delete, FALSE);
	 gtk_widget_set_sensitive (change, TRUE);
      }
   }

   if (!text[0] || !*text[0] || !text[1] || !*text[1]) {
      gtk_widget_set_sensitive (add, FALSE);
      gtk_widget_set_sensitive (change, FALSE);
   } else {
      gtk_widget_set_sensitive (add, TRUE);
      if (row >= 0)
	 gtk_widget_set_sensitive (change, TRUE);
   }

   /* new button */
   if ((!text[0] || !*text[0]) && (!text[1] || !*text[1]))
      gtk_widget_set_sensitive (new, FALSE);
   else
      gtk_widget_set_sensitive (new, TRUE);
}


static gboolean
check_duplicate_comment_key (const gchar *key, gint this_row)
{
   GtkCList *clist = GTK_CLIST (prefs_win.comment_clist);
   gint row;

   g_return_val_if_fail (key && *key, FALSE);

   for (row = 0; row < clist->rows; row++) {
      CommentDataEntry *entry;

      if (row == this_row) continue;

      entry = gtk_clist_get_row_data (clist, row);
      if (!entry) continue;

      if (!strcmp (entry->key, key)) {
	 /* FIXME!! add error handling */
	 return TRUE;
      }
   }

   return FALSE;
}


void
cb_comment_select_row (GtkCList *clist,
		       gint row, gint col,
		       GdkEventButton *event,
		       gpointer data)
{
   GtkToggleButton *t_ins  = GTK_TOGGLE_BUTTON (prefs_win.comment_status_toggle);
   GtkToggleButton *t_auto = GTK_TOGGLE_BUTTON (prefs_win.comment_auto_toggle);
   GtkToggleButton *t_disp = GTK_TOGGLE_BUTTON (prefs_win.comment_disp_toggle);
   GtkEntry *key_entry  = GTK_ENTRY (prefs_win.comment_key_entry);
   GtkEntry *name_entry = GTK_ENTRY (prefs_win.comment_name_entry);
   CommentDataEntry *entry;

   prefs_win.comment_selected = row;

   entry = gtk_clist_get_row_data (clist, row);
   if (!entry) return;

   gtk_toggle_button_set_active (t_ins,  entry->enable);
   gtk_toggle_button_set_active (t_auto, entry->auto_val);
   gtk_toggle_button_set_active (t_disp, entry->display);

   gtk_entry_set_text (key_entry, entry->key);
   if (entry->userdef)
      gtk_entry_set_text (name_entry, entry->display_name);
   else
      gtk_entry_set_text (name_entry, _(entry->display_name));

   set_sensitive_comment ();
}


void
cb_comment_unselect_row (GtkCList *clist,
			 gint row, gint col,
			 GdkEventButton *event,
			 gpointer data)
{
   prefs_win.comment_selected = -1;

   set_sensitive_comment ();
}


static void
clear_comment_entry ()
{
   GtkCList *clist  = GTK_CLIST (prefs_win.comment_clist);
   GtkEntry *entry1 = GTK_ENTRY (prefs_win.comment_key_entry);
   GtkEntry *entry2 = GTK_ENTRY (prefs_win.comment_name_entry);
   GtkToggleButton *t_ins  = GTK_TOGGLE_BUTTON (prefs_win.comment_status_toggle);
   GtkToggleButton *t_auto = GTK_TOGGLE_BUTTON (prefs_win.comment_auto_toggle);
   GtkToggleButton *t_disp = GTK_TOGGLE_BUTTON (prefs_win.comment_disp_toggle);

   gtk_clist_unselect_all (clist);
   gtk_entry_set_text (entry1, "\0");
   gtk_entry_set_text (entry2, "\0");

   gtk_toggle_button_set_active (t_ins,  TRUE);
   gtk_toggle_button_set_active (t_auto, FALSE);
   gtk_toggle_button_set_active (t_disp, TRUE);
}


static void
cb_comment_new_button_pressed (GtkButton *button, gpointer data)
{
   clear_comment_entry ();
   set_sensitive_comment ();
}


static void
cb_comment_add_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.comment_clist);
   GtkToggleButton *t_ins  = GTK_TOGGLE_BUTTON (prefs_win.comment_status_toggle);
   GtkToggleButton *t_auto = GTK_TOGGLE_BUTTON (prefs_win.comment_auto_toggle);
   GtkToggleButton *t_disp = GTK_TOGGLE_BUTTON (prefs_win.comment_disp_toggle);
   GtkEntry *entry1 = GTK_ENTRY (prefs_win.comment_key_entry);
   GtkEntry *entry2 = GTK_ENTRY (prefs_win.comment_name_entry);
   CommentDataEntry *rowdata;
   const gchar *key, *name;
   gchar *text[16], message[BUF_SIZE];
   gint row;

   key  = gtk_entry_get_text (entry1);
   name = gtk_entry_get_text (entry2);

   if (!key || !*key) {
      g_snprintf (message, BUF_SIZE, "\"Key name\" must be defined.");
      gtkutil_message_dialog ("Error!!", message);
      return;
   }
   if (!name || !*name) {
      g_snprintf (message, BUF_SIZE, "\"Display name\" must be defined.");
      gtkutil_message_dialog ("Error!!", message);
      return;
   }

   if (check_duplicate_comment_key (key, -1)) {
      g_snprintf (message, BUF_SIZE, "\"%s\" is already defined.", key);
      gtkutil_message_dialog ("Error!!", message);
      return;
   }

   rowdata = g_new0 (CommentDataEntry, 1);
   rowdata->key          = g_strdup (key);
   rowdata->display_name = g_strdup (name);
   rowdata->value        = NULL;
   rowdata->enable       = t_ins->active;
   rowdata->auto_val     = t_auto->active;
   rowdata->display      = t_disp->active;
   rowdata->userdef      = TRUE;

   text[0] = rowdata->key;
   text[1] = rowdata->display_name;
   text[2] = _(boolean_to_text (rowdata->enable));
   text[3] = _(boolean_to_text (rowdata->auto_val));
   text[4] = _(boolean_to_text (rowdata->display));
   text[5] = _("User defined");
   row = gtk_clist_append (clist, text);

   if (row >= 0) {
      gtk_clist_set_row_data_full (clist, row, rowdata,
				   (GtkDestroyNotify) comment_data_entry_delete);
      sync_to_row_data_comment (row);
   } else {
      comment_data_entry_delete (rowdata);;
   }

   update_config_comment ();
   clear_comment_entry ();
   set_sensitive_comment ();
}


static void
cb_comment_change_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.comment_clist);
   GtkToggleButton *t_ins  = GTK_TOGGLE_BUTTON (prefs_win.comment_status_toggle);
   GtkToggleButton *t_auto = GTK_TOGGLE_BUTTON (prefs_win.comment_auto_toggle);
   GtkToggleButton *t_disp = GTK_TOGGLE_BUTTON (prefs_win.comment_disp_toggle);
   GtkEntry *entry1 = GTK_ENTRY (prefs_win.comment_key_entry);
   GtkEntry *entry2 = GTK_ENTRY (prefs_win.comment_name_entry);
   CommentDataEntry *entry;
   gint row = prefs_win.comment_selected;
   const gchar *key, *name;
   gchar message[BUF_SIZE];

   key  = gtk_entry_get_text (entry1);
   name = gtk_entry_get_text (entry2);

   if (!key || !*key) {
      g_snprintf (message, BUF_SIZE, "\"Key name\" must be defined.");
      gtkutil_message_dialog ("Error!!", message);
      return;
   }
   if (!name || !*name) {
      g_snprintf (message, BUF_SIZE, "\"Display name\" must be defined.");
      gtkutil_message_dialog ("Error!!", message);
      return;
   }

   entry = gtk_clist_get_row_data (clist, row);
   if (!entry) return;

   if (!entry->userdef) {
      if (strcmp (entry->key, key)) {
	 /* FIXME!! add error handling */
	 return;
      }
      if (strcmp (_(entry->display_name), name)) {
	 /* FIXME!! add error handling */
	 return;
      }

      if (!entry->def_val_fn)
	 entry->auto_val = FALSE;
      else
	 entry->auto_val = t_auto->active;
   } else {
      if (check_duplicate_comment_key (key, row)) {
	 g_snprintf (message, BUF_SIZE, "\"%s\" is already defined.", key);
	 gtkutil_message_dialog ("Error!!", message);
 	 return;
      }

      g_free (entry->key);
      g_free (entry->display_name);
      entry->key          = g_strdup (key);
      entry->display_name = g_strdup (name);

      entry->auto_val = FALSE;
   }

   entry->enable  = t_ins->active;
   entry->display = t_disp->active;

   sync_to_row_data_comment (row);
   update_config_comment ();
   set_sensitive_comment ();
}


static void
cb_comment_del_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.comment_clist);
   CommentDataEntry *entry;
   gint row = prefs_win.comment_selected;

   if (row < 0 || row > clist->rows) return;

   entry = gtk_clist_get_row_data (clist, row);
   if (!entry) return;
   if (!entry->userdef) return;

   gtk_clist_remove (clist, row);

   update_config_comment ();
   clear_comment_entry ();
   set_sensitive_comment ();
}


static void
cb_comment_key_entry_changed (GtkEditable *entry, gpointer data)
{
   set_sensitive_comment ();
}


static void
cb_comment_name_entry_changed (GtkEditable *entry, gpointer data)
{
   set_sensitive_comment ();
}


static void
cb_comment_charset_changed (GtkEditable *entry, gpointer data)
{
   if (config_changed->comment_charset != config_prechanged->comment_charset)
      g_free (config_changed->comment_charset);

   config_changed->comment_charset
      = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
}


/*******************************************************************************
 *  prefs_cache_page:
 *     @ Create cache preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_cache_page ()
{
   GtkWidget *main_vbox;
   GtkWidget *frame, *frame_vbox, *vbox, *hbox, *table;
   GtkWidget *scrollwin, *hseparator;
   GtkWidget *clist;
   GtkWidget *button1, *button2;
   GtkWidget *label, *menu, *menu_item, *option_menu;
   GList *list, *node;
   gint i, item = 0;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /**********************************************
    *  Cache reading frame
    **********************************************/
   prefs_create_frame(_("Cache reading"), frame, frame_vbox, main_vbox);

   /* clist */
   table = gtk_table_new (2, 3, FALSE);
   gtk_container_add (GTK_CONTAINER (frame_vbox), table);

   /* possible columns */
   label = gtk_label_new (_("Available cache type"));
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
		     0, 0, 0, 0);

   scrollwin = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_set_border_width(GTK_CONTAINER(scrollwin), 5);
   gtk_table_attach (GTK_TABLE (table), scrollwin, 0, 1, 1, 2,
		     GTK_FILL | GTK_EXPAND, GTK_EXPAND, 0, 0);
   gtk_widget_set_usize (scrollwin, -1, 200);

   clist = prefs_win.cache_read_clist1 =  gtk_clist_new (1);
   gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
   gtk_clist_set_auto_sort (GTK_CLIST (clist), TRUE);
   gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 0, TRUE);
   gtk_container_add (GTK_CONTAINER (scrollwin), clist);

   list = node = thumbsupport_get_loader_list ();
   while (node) {
      gchar *text = node->data;
      gtk_clist_append (GTK_CLIST (clist), &text);
      node = g_list_next (node);
   }
   g_list_free (list);

   gtk_signal_connect (GTK_OBJECT (clist),"select_row",
		       GTK_SIGNAL_FUNC (cb_clist_cache_read_select_row),
		       &prefs_win.cache_read_clist1_selected);
   gtk_signal_connect (GTK_OBJECT (clist),"unselect_row",
		       GTK_SIGNAL_FUNC (cb_clist_cache_read_unselect_row),
		       &prefs_win.cache_read_clist1_selected);

   prefs_win.cache_read_clist1_selected = -1;

   /* add/delete buttons */
   vbox = gtk_vbox_new (TRUE, 0);
   /* gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0); */
   gtk_table_attach (GTK_TABLE (table), vbox, 1, 2, 1, 2,
		     GTK_FILL, GTK_FILL, 0, 0);

   button1 = gtk_button_new_with_label (_("Add"));
   prefs_win.cache_read_add = button1;
   gtk_widget_set_usize (button1, 50, -1);
   gtk_box_pack_start (GTK_BOX (vbox), button1, FALSE, FALSE, 2);

   button2 = gtk_button_new_with_label (_("Delete"));
   prefs_win.cache_read_del = button2;
   gtk_widget_set_usize (button2, 50, -1);
   gtk_box_pack_start (GTK_BOX (vbox), button2, FALSE, FALSE, 2);

   gtk_signal_connect (GTK_OBJECT (button1),"released",
		       GTK_SIGNAL_FUNC (cb_cache_read_add_button_pressed), clist);
   gtk_signal_connect (GTK_OBJECT (button2),"released",
		       GTK_SIGNAL_FUNC (cb_cache_read_del_button_pressed), clist);

   hseparator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (vbox), hseparator, FALSE, FALSE, 2);

   /* move buttons */
   button1 = gtk_button_new_with_label (_("Up"));
   prefs_win.cache_read_up = button1;
   gtk_widget_set_usize (button1, 50, -1);
   gtk_box_pack_start (GTK_BOX (vbox), button1, FALSE, FALSE, 2);

   button2 = gtk_button_new_with_label (_("Down"));
   prefs_win.cache_read_down = button2;
   gtk_widget_set_usize (button2, 50, -1);
   gtk_box_pack_start (GTK_BOX (vbox), button2, FALSE, FALSE, 2);

   /* Use cache type */
   label = gtk_label_new (_("Cache type to use"));
   gtk_table_attach (GTK_TABLE (table), label, 2, 3, 0, 1,
		     0, 0, 0, 0);

   scrollwin = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_set_border_width(GTK_CONTAINER(scrollwin), 5);
   /* gtk_box_pack_start (GTK_BOX (hbox), scrollwin, TRUE, TRUE, 0); */
   gtk_table_attach (GTK_TABLE (table), scrollwin, 2, 3, 1, 2,
		     GTK_FILL | GTK_EXPAND, GTK_EXPAND, 0, 0);
   gtk_widget_set_usize (scrollwin, -1, 200);

   clist = prefs_win.cache_read_clist2 = gtk_clist_new (1);
   gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
   gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 0, TRUE);
   gtk_container_add (GTK_CONTAINER (scrollwin), clist);

   gtk_signal_connect (GTK_OBJECT (clist),"select_row",
		       GTK_SIGNAL_FUNC (cb_clist_cache_read_select_row),
		       &prefs_win.cache_read_clist2_selected);
   gtk_signal_connect (GTK_OBJECT (clist),"unselect_row",
		       GTK_SIGNAL_FUNC (cb_clist_cache_read_unselect_row),
		       &prefs_win.cache_read_clist2_selected);

   prefs_win.cache_read_clist2_selected = -1;
   cache_read_set_data (conf.cache_read_list);

   gtk_signal_connect (GTK_OBJECT (button1),"released",
		       GTK_SIGNAL_FUNC (cb_cache_read_up_button_pressed),
		       NULL);
   gtk_signal_connect (GTK_OBJECT (button2),"released",
		       GTK_SIGNAL_FUNC (cb_cache_read_down_button_pressed),
		       NULL);

   /**********************************************
    *  Cache writing frame
    **********************************************/
   prefs_create_frame(_("Cache writing"), frame, vbox, main_vbox);

   /* Cache write type menu */
   list = node = thumbsupport_get_saver_list ();
   /* list = node = g_list_prepend (list, "NONE"); */

   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

   label = gtk_label_new (_("Cache type for save"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
   option_menu = prefs_win.cache_write_type = gtk_option_menu_new();
   menu = gtk_menu_new();
   for (i = 0; node; i++) {
      gchar *text = node->data;

      if (!strcmp(text, conf.cache_write_type)) item = i;
      menu_item = gtk_menu_item_new_with_label (_(text));
      gtk_object_set_data (GTK_OBJECT (menu_item), "label", text);
      gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			 GTK_SIGNAL_FUNC(cb_cache_write_type), text);
      gtk_menu_append (GTK_MENU(menu), menu_item);
      gtk_widget_show (menu_item);
      node = g_list_next (node);
   }
   gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
   gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu),
				item);
   gtk_box_pack_start (GTK_BOX (hbox), option_menu, FALSE, FALSE, 5);

   button1 = gtk_button_new_with_label (_("Preference"));
   prefs_win.cache_write_prefs = button1;
   gtk_box_pack_start (GTK_BOX (hbox), button1, FALSE, FALSE, 0);
   gtk_signal_connect_object (GTK_OBJECT (button1),"clicked",
			      GTK_SIGNAL_FUNC (cb_cache_write_prefs_button_pressed),
			      GTK_OBJECT (option_menu));

   g_list_free (list);

   return main_vbox;
}


/*******************************************************************************
 *  prefs_comment_page:
 *     @ Create comment preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_comment_page ()
{
   GtkWidget *main_vbox, *frame, *frame_vbox, *hbox, *hbox1, *vbox;
   GtkWidget *scrollwin, *clist, *entry, *toggle, *button, *label, *combo;
   gchar *titles[] = {
      N_("Key Name"),
      N_("Displayed Name"),
      N_("Status"),
      N_("Auto"),
      N_("Display"),
      NULL
   };
   gint i, titles_num = sizeof (titles) / sizeof (gchar *);

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /********************************************** 
    * Key Name definition frame
    **********************************************/
   prefs_create_frame(_("Key Name Definition"), frame, frame_vbox, main_vbox);

   scrollwin = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_set_border_width(GTK_CONTAINER(scrollwin), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), scrollwin, TRUE, TRUE, 0);
   gtk_widget_set_usize (scrollwin, -1, 250);

   clist = prefs_win.comment_clist = gtk_clist_new (6);
   gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
   /* gtk_clist_set_auto_sort (GTK_CLIST (clist), TRUE); */
   gtk_container_add (GTK_CONTAINER (scrollwin), clist);
   gtk_widget_show (clist);

   for (i = 0; i < titles_num; i++) {
      gtk_clist_set_column_title (GTK_CLIST (clist), i, _(titles[i]));
      gtk_clist_set_column_auto_resize (GTK_CLIST (clist), i, TRUE);
   }
   gtk_clist_column_titles_show (GTK_CLIST (clist));

   gtk_signal_connect (GTK_OBJECT (clist),"select_row",
		       GTK_SIGNAL_FUNC (cb_comment_select_row),
		       NULL);
   gtk_signal_connect (GTK_OBJECT (clist),"unselect_row",
		       GTK_SIGNAL_FUNC (cb_comment_unselect_row),
		       NULL);

   prefs_win.comment_selected = -1;

   /* entry area */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   vbox = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);

   label = gtk_label_new (_("Key Name: "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);

   entry = prefs_win.comment_key_entry = gtk_entry_new ();
   /* gtk_widget_set_usize (entry, 100, -1); */
   gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (entry),"changed",
		       GTK_SIGNAL_FUNC (cb_comment_key_entry_changed), NULL);

   vbox = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);

   label = gtk_label_new (_("Displayed Name: "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);

   entry = prefs_win.comment_name_entry = gtk_entry_new ();
   gtk_box_pack_start (GTK_BOX (hbox1), entry, TRUE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (entry),"changed",
		       GTK_SIGNAL_FUNC (cb_comment_name_entry_changed), NULL);

   /* "insert" check box */
   toggle = create_check_button (_("Enable"), FALSE, NULL, NULL);
   prefs_win.comment_status_toggle = toggle;
   gtk_box_pack_start (GTK_BOX (hbox1), toggle, FALSE, FALSE, 0);

   /* "Auto" check box */
   toggle = create_check_button (_("Auto"), FALSE, NULL, NULL);
   prefs_win.comment_auto_toggle = toggle;
   gtk_box_pack_start (GTK_BOX (hbox1), toggle, FALSE, FALSE, 0);

   /* "display" check box */
   toggle = create_check_button (_("Display"), FALSE, NULL, NULL);
   prefs_win.comment_disp_toggle = toggle;
   gtk_box_pack_start (GTK_BOX (hbox1), toggle, FALSE, FALSE, 0);

   /* edit buttons */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_end (GTK_BOX (hbox), hbox1, FALSE, TRUE, 0);
   gtk_box_set_homogeneous (GTK_BOX(hbox1), TRUE);

   button = prefs_win.comment_new = gtk_button_new_with_label (_("New"));
   gtk_widget_set_usize (button, 70, -1);
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"clicked",
		       GTK_SIGNAL_FUNC (cb_comment_new_button_pressed), clist);

   button = prefs_win.comment_add = gtk_button_new_with_label (_("Add"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"clicked",
		       GTK_SIGNAL_FUNC (cb_comment_add_button_pressed), clist);

   button = prefs_win.comment_change = gtk_button_new_with_label (_("Change"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"clicked",
		       GTK_SIGNAL_FUNC (cb_comment_change_button_pressed), clist);

   button = prefs_win.comment_del = gtk_button_new_with_label (_("Delete"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"clicked",
		       GTK_SIGNAL_FUNC (cb_comment_del_button_pressed), clist);

   /********************************************** 
    * Charset Frame
    **********************************************/
   prefs_create_frame (_("Character set"), frame, frame_vbox, main_vbox);

   /* locale charset */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, FALSE, TRUE, 5);
   label = gtk_label_new (_("Character set: "));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 5);
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   combo = gtk_combo_new ();
   gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, TRUE, 5);
   gtk_combo_set_popdown_strings (GTK_COMBO (combo),
				  charset_get_known_list(NULL));
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry),
		       conf.comment_charset);
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO (combo)->entry), "changed",
		       GTK_SIGNAL_FUNC (cb_comment_charset_changed), NULL);


   set_value_comment ();

   return main_vbox;
}


/*******************************************************************************
 *  prefs_search_page:
 *     @ Create search preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_search_page ()
{
   GtkWidget *main_vbox, *frame, *frame_vbox, *hbox;
   GtkWidget *label, *spinner;
   GtkAdjustment *adj;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /**********************************************
    *  Find duplicates
    **********************************************/
   prefs_create_frame(_("Find duplicates"), frame, frame_vbox, main_vbox);

   /* Accuracy Spinner */
   hbox = gtk_hbox_new (FALSE, 5);
   gtk_container_set_border_width (GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, FALSE, FALSE, 0);
   label = gtk_label_new (_("Accuracy"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   adj = (GtkAdjustment *) gtk_adjustment_new (conf.search_similar_accuracy,
					       0.0, 1.0, 0.01, 0.1, 0.0);
   spinner = gtk_spin_button_new (adj, 0, 0);
   gtk_spin_button_set_digits (GTK_SPIN_BUTTON (spinner), 2);
   gtk_widget_set_usize(spinner, 50, -1);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner), TRUE);
   gtk_spin_button_set_shadow_type (GTK_SPIN_BUTTON (spinner), GTK_SHADOW_OUT);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
		       GTK_SIGNAL_FUNC (cb_get_data_from_adjustment_by_float),
		       &config_changed->search_similar_accuracy);
   gtk_box_pack_start (GTK_BOX (hbox), spinner, FALSE, FALSE, 0);

   return main_vbox;
}


/*******************************************************************************
 *  prefs_slideshow_page:
 *     @ Create slide show preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_slideshow_page ()
{
   GtkWidget *main_vbox, *frame;
   GtkWidget *hbox, *vbox;
   GtkWidget *label;
   GtkWidget *spinner, *toggle, *button;
   GtkAdjustment *adj;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   prefs_create_frame(NULL, frame, vbox, main_vbox);

   /* Default Image Scale Spinner */
   hbox = gtk_hbox_new (FALSE, 5);
   gtk_container_set_border_width (GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   label = gtk_label_new (_("Image change interval"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   adj = (GtkAdjustment *) gtk_adjustment_new (conf.slideshow_interval,
					       0.0, 7200.0, 0.01, 0.1, 0.0);
   spinner = gtk_spin_button_new (adj, 0, 0);
   gtk_widget_set_usize(spinner, 70, -1);
   gtk_spin_button_set_digits (GTK_SPIN_BUTTON (spinner), 2);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner), TRUE);
   gtk_spin_button_set_shadow_type (GTK_SPIN_BUTTON (spinner), GTK_SHADOW_OUT);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
		       GTK_SIGNAL_FUNC (cb_get_data_from_adjustment_by_float),
		       &config_changed->slideshow_interval);
   gtk_box_pack_start (GTK_BOX (hbox), spinner, FALSE, FALSE, 0);

   label = gtk_label_new (_("[sec]"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

   /* repeat */
   toggle = create_check_button (_("Repeat slide show"), conf.slideshow_repeat,
				 cb_get_data_from_toggle,
				 &config_changed->slideshow_repeat);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* fit to window */
   toggle = create_check_button (_("Auto resize to window size"), conf.slideshow_fit_to_win,
				 cb_get_data_from_toggle,
				 &config_changed->slideshow_fit_to_win);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);


   /********************************************** 
    * Show/Hide Frame
    **********************************************/
   prefs_create_frame(_("Show/Hide"), frame, vbox, main_vbox);

   /* Show Menubar or not */
   toggle = create_check_button (_("Show menubar"), conf.slideshow_menubar,
				 cb_get_data_from_toggle,
				 &config_changed->slideshow_menubar);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Show Toolbar or not */
   toggle = create_check_button (_("Show toolbar"), conf.slideshow_toolbar,
				 cb_get_data_from_toggle,
				 &config_changed->slideshow_toolbar);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Show Player toolbar or not */
   toggle = create_check_button (_("Show player toolbar"), conf.slideshow_player,
				 cb_get_data_from_toggle,
				 &config_changed->slideshow_player);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Show Statusbar or not */
   toggle = create_check_button (_("Show statusbar"), conf.slideshow_statusbar,
				 cb_get_data_from_toggle,
				 &config_changed->slideshow_statusbar);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);


   /********************************************** 
    * Back Ground Frame
    **********************************************/
   prefs_create_frame(_("Back Ground"), frame, vbox, main_vbox);

   hbox = gtk_hbox_new (FALSE, 5);
   gtk_container_set_border_width (GTK_CONTAINER(hbox), 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

   toggle = create_check_button (_("Use specified color"), conf.slideshow_set_bg,
				 cb_get_data_from_toggle,
				 &config_changed->slideshow_set_bg);
   gtk_box_pack_start (GTK_BOX (hbox), toggle, FALSE, FALSE, 0);

   button = gtk_button_new_with_label (_("Choose Color"));
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (button),"clicked",
		       GTK_SIGNAL_FUNC (cb_choose_color),
		       config_changed->slideshow_bg_color);

   return main_vbox;
}


/*******************************************************************************
 *  prefs_dnd_page:
 *     @ Create Drag and Drop preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_dnd_page ()
{
   GtkWidget *main_vbox;
   GtkWidget *frame, *vbox;
   GtkWidget *toggle;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /**********************************************
    *  File Operation frame
    **********************************************/
   prefs_create_frame(_("File operation"), frame, vbox, main_vbox);

   /* Drag and Drop to external proccess */
   toggle = create_check_button (_("Enable DnD to external proccess (Experimental)"),
				 conf.dnd_enable_to_external,
				 cb_get_data_from_toggle,
				 &config_changed->dnd_enable_to_external);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Drag and Drop from external proccess */
   toggle = create_check_button (_("Enable DnD from extenal proccess (Experimental)"),
				 conf.dnd_enable_from_external,
				 cb_get_data_from_toggle,
				 &config_changed->dnd_enable_from_external);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   /* Always refresh list when DnD end */
   toggle = create_check_button (_("Always refresh list when DnD end"),
				 conf.dnd_refresh_list_always,
				 cb_get_data_from_toggle,
				 &config_changed->dnd_refresh_list_always);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);

   return main_vbox;
}


void
prefs_ui_etc_set_sensitive ()
{
   set_sensitive_cache_read ();
   set_sensitive_cache_write ();
   set_sensitive_comment ();
}
