/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __THUMBNAIL_H__
#define __THUMBNAIL_H__

#include "gimageview.h"
#include "gimv_image.h"
#include "fileload.h"
#include "fr-archive.h"
#include "image_info.h"
#include "thumbnail_support.h"
#include "thumbnail_view.h"

#define ICON_SIZE 18

struct Thumbnail_Tag
{
   ImageInfo      *info;

   ThumbView      *thumb_view;
   GHashTable     *mode_data;

   gboolean        selected;
   gboolean        sync;

   /* for thumbnail */
   GimvImage      *image;
   GdkPixmap      *thumbnail;
   GdkBitmap      *thumbnail_mask;
   GdkPixmap      *icon;
   GdkBitmap      *icon_mask;
   gchar          *cache_type;
   gint            thumb_width;
   gint            thumb_height;

   guint           ref_count;
};


Thumbnail     *thumbnail_new                  (const gchar    *filename);
Thumbnail     *thumbnail_new_with_image_info  (ImageInfo      *info);
ThumbView     *thumbnail_get_parent_thumbview (Thumbnail      *thumb);
void           thumbnail_set_parent_thumbview (Thumbnail      *thumb,
					       ThumbView      *tv);
void           thumbnail_finalize             (Thumbnail      *thumb);
Thumbnail     *thumbnail_ref                  (Thumbnail      *thumb);
void           thumbnail_unref                (Thumbnail      *thumb);
gchar         *thumbnail_get_image_name       (Thumbnail      *thumb);
gboolean       thumbnail_create               (Thumbnail      *thumb,
					       gint            thumb_size,
					       ThumbLoadType   type);
void           thumbnail_get_thumb            (Thumbnail      *thumb,
					       GdkPixmap     **pixmap,
					       GdkBitmap     **mask);
GtkWidget     *thumbnail_get_thumb_by_widget  (Thumbnail      *thumb);
void           thumbnail_get_icon             (Thumbnail      *thumb,
					       GdkPixmap     **pixmap,
					       GdkBitmap     **mask);
GtkWidget     *thumbnail_get_icon_by_widget   (Thumbnail      *thumb);
gchar         *thumbnail_get_cache_type       (Thumbnail      *thumb);
gchar         *thumbnail_extract_archive      (Thumbnail      *thumb);
gboolean       thumbnail_rename_image         (Thumbnail      *thumb,
					       const gchar    *filename);
gboolean       thumbnail_has_thumbnail        (Thumbnail      *thumb);
gboolean       thumbnail_is_dir               (Thumbnail      *thumb);
gboolean       thumbnail_is_archive           (Thumbnail      *thumb);
gboolean       thumbnail_is_in_archive        (Thumbnail      *thumb);
gchar         *thumbnail_find_thumbcache      (const gchar    *filename,
					       gchar         **type);

#endif /* __THUMBNAIL_H__ */
